@@ -1,10 +1,8 @@
-import sys
-
 from sqlalchemy.testing.requirements import Requirements
 
 from alembic import util
-from alembic.testing import exclusions
 from alembic.util import sqla_compat
+from ..testing import exclusions
 
 
 class SuiteRequirements(Requirements):
@@ -21,6 +19,11 @@ class SuiteRequirements(Requirements):
 
         return exclusions.closed()
 
+    @property
+    def materialized_views(self):
+        """needed for sqlalchemy compat"""
+        return exclusions.closed()
+
     @property
     def unique_constraint_reflection(self):
         def doesnt_have_check_uq_constraints(config):
@@ -52,6 +55,10 @@ class SuiteRequirements(Requirements):
     def foreign_key_match(self):
         return exclusions.open()
 
+    @property
+    def foreign_key_constraint_reflection(self):
+        return exclusions.open()
+
     @property
     def check_constraints_w_enforcement(self):
         """Target database must support check constraints
@@ -82,15 +89,18 @@ class SuiteRequirements(Requirements):
         )
 
     @property
-    def python3(self):
+    def sqlalchemy_1x(self):
         return exclusions.skip_if(
-            lambda: sys.version_info < (3,), "Python version 3.xx is required."
+            lambda config: not util.sqla_1x,
+            "SQLAlchemy 1.x test",
         )
 
     @property
-    def pep3147(self):
-
-        return exclusions.only_if(lambda config: util.compat.has_pep3147())
+    def sqlalchemy_2(self):
+        return exclusions.skip_if(
+            lambda config: not util.sqla_2,
+            "SQLAlchemy 2.x test",
+        )
 
     @property
     def comments(self):
@@ -112,6 +122,74 @@ class SuiteRequirements(Requirements):
             exclusions.BooleanPredicate(sqla_compat.has_computed)
         )
 
+    @property
+    def computed_reflects_normally(self):
+        return exclusions.only_if(
+            exclusions.BooleanPredicate(sqla_compat.has_computed_reflection)
+        )
+
+    @property
+    def computed_reflects_as_server_default(self):
+        return exclusions.closed()
+
+    @property
+    def computed_doesnt_reflect_as_server_default(self):
+        return exclusions.closed()
+
+    @property
+    def autoincrement_on_composite_pk(self):
+        return exclusions.closed()
+
+    @property
+    def fk_ondelete_is_reflected(self):
+        return exclusions.closed()
+
+    @property
+    def fk_onupdate_is_reflected(self):
+        return exclusions.closed()
+
+    @property
+    def fk_onupdate(self):
+        return exclusions.open()
+
+    @property
+    def fk_ondelete_restrict(self):
+        return exclusions.open()
+
+    @property
+    def fk_onupdate_restrict(self):
+        return exclusions.open()
+
+    @property
+    def fk_ondelete_noaction(self):
+        return exclusions.open()
+
+    @property
+    def fk_initially(self):
+        return exclusions.closed()
+
+    @property
+    def fk_deferrable(self):
+        return exclusions.closed()
+
+    @property
+    def fk_deferrable_is_reflected(self):
+        return exclusions.closed()
+
+    @property
+    def fk_names(self):
+        return exclusions.open()
+
+    @property
+    def integer_subtype_comparisons(self):
+        return exclusions.open()
+
+    @property
+    def no_name_normalize(self):
+        return exclusions.skip_if(
+            lambda config: config.db.dialect.requires_name_normalize
+        )
+
     @property
     def identity_columns(self):
         return exclusions.closed()
