@@ -5,7 +5,10 @@ from unittest.mock import patch
 
 from dateutil import tz
 import sqlalchemy as sa
+from sqlalchemy import Column
 from sqlalchemy import inspect
+from sqlalchemy import MetaData
+from sqlalchemy import Table
 
 from alembic import autogenerate
 from alembic import command
@@ -707,7 +710,7 @@ class ImportsTest(TestBase):
                 context.configure(
                     connection=connection,
                     target_metadata=target_metadata,
-                    **kw
+                    **kw,
                 )
                 with context.begin_transaction():
                     context.run_migrations()
@@ -717,7 +720,6 @@ class ImportsTest(TestBase):
         )
 
     def test_imports_in_script(self):
-        from sqlalchemy import MetaData, Table, Column
         from sqlalchemy.dialects.mysql import VARCHAR
 
         type_ = VARCHAR(20, charset="utf8", national=True)
